try:
    # get_args was added in 3.8
    from typing import get_args

except ImportError:
    # to support python 3.7
    from typing_extensions import get_args


# local imports
from .type_definitions import LOCATIONS, GeographicScale, GeographicContext, Year


def validate_location(location: str) -> str:
    location_key = LOCATIONS.get(location.lower())  # noqa

    if location_key is None:
        valid_locations = sorted(list(LOCATIONS.keys.union(LOCATIONS.values)))
        error_message = f"Invalid location: {location}. Valid location values are\n{valid_locations}"
        raise ValueError(error_message)

    return location_key


def validate_geographic_scale(geographic_scale: GeographicScale) -> str:
    valid_geo_scales = get_args(GeographicScale)

    if geographic_scale not in valid_geo_scales:
        valid_geo_scales = sorted(valid_geo_scales)
        error_message = f"Invalid geographic scale: {geographic_scale}. Valid geographic scale values are\n{valid_geo_scales}"
        raise ValueError(error_message)

    return geographic_scale


def validate_geographic_context(geographic_context: GeographicContext) -> str:
    valid_geo_contexts = get_args(GeographicContext)

    if geographic_context not in valid_geo_contexts:
        valid_geo_scales = sorted(valid_geo_contexts)
        error_message = f"Invalid geographic context: {geographic_context}. Valid geographic context values are\n{valid_geo_scales}"
        raise ValueError(error_message)

    return geographic_context


def validate_year(year: Year) -> str:
    year_str = str(year)

    valid_years = get_args(Year)
    if year_str not in valid_years:
        error_message = f"Invalid year: {year}. Valid year values are\n{valid_years}"
        raise ValueError(error_message)

    return year_str



from types import MappingProxyType

_BASE_URL = "https://services3.arcgis.com/ZvidGQkLaDJxRSJ2/ArcGIS/rest/services"

US_COUNTY_FEATURE_SERVER_URLS = MappingProxyType(
    {
        "2000": f"{_BASE_URL}/Overall_2000_Counties/FeatureServer/0",
        "2010": f"{_BASE_URL}/Overall_2010_Counties/FeatureServer/0",
        "2014": f"{_BASE_URL}/Overall_2014_Counties/FeatureServer/1",
        "2016": f"{_BASE_URL}/Overall_2016_Counties/FeatureServer/0",
        "2018": f"{_BASE_URL}/CDC_Social_Vulnerability_Index_2018/FeatureServer/1",
    }
)

US_TRACT_FEATURE_SERVERS_URLS = MappingProxyType(
    {
        "2000": f"{_BASE_URL}/Overall_2000_Tracts/FeatureServer/0",
        "2010": f"{_BASE_URL}/Overall_2010_Tracts/FeatureServer/0",
        "2014": f"{_BASE_URL}/Overall_2014_Tracts/FeatureServer/1",
        "2016": f"{_BASE_URL}/Overall_2016_Tracts/FeatureServer/0",
        "2018": f"{_BASE_URL}/CDC_Social_Vulnerability_Index_2018/FeatureServer/2",
    }
)

# Developer documentation

# 2000:
# counties:
#     US: Overall_2000_Counties/FeatureServer/0
# tracts:
#     US: Overall_2000_Tracts/FeatureServer/0

# 2010:
# counties:
#     US: Overall_2010_Counties/FeatureServer/0
# tracts:
#     US: Overall_2010_Tracts/FeatureServer/0

# 2014:
# counties:
#     US: Overall_2014_Counties/FeatureServer/1
# tracts:
#     US: Overall_2014_Tracts/FeatureServer/1
#     # NOTE: all STATE and COUNTY fields are prefixed with a space
#     # use STATE LIKE '%%' to get around this

# 2016:
# counties:
#     US: Overall_2016_Counties/FeatureServer/0
# tracts:
#     US: Overall_2016_Tracts/FeatureServer/0

# 2018:
# counties:
#     US: CDC_Social_Vulnerability_Index_2018/FeatureServer/1
# tracts:
#     US: CDC_Social_Vulnerability_Index_2018/FeatureServer/2

# cols to support:
# state_fips: State FIPS code
# state_name: State name
# county_name: County name
# fips: Census tract or county fips code
# svi_edition: year corresponding to svi release (this assumes 2 SVI's will not be release in a given year in the future)
# geometry: County or census tract simple features geometry
# rank_theme_1: Socioeconomic
# rank_theme_2: Household Composition / Disability
# rank_theme_3: Minority Status / Language
# rank_theme_4: Housing Type / Transportation
# rank_svi: aggregated overall percentile ranking
# value_theme_1: Socioeconomic
# value_theme_2: Household Composition / Disability
# value_theme_3: Minority Status / Language
# value_theme_4: Housing Type / Transportation
# value_svi: aggregated overall value; sum of values from themes 1, 2, 3, 4.

# state_fips:
#     counties: STATE_FIPS, FIRST_STATE_FIPS, ST, ST, ST
#     tracts: STATE_FIPS, STATE_FIPS, ST, ST, ST
# county_fips:
#     counties: CNTY_FIPS, FIRST_CNTY_FIPS, FIPS[len(ST):], FIPS[len(ST):], FIPS[len(ST):]
#     tracts: CNTY_FIPS, CNTY_FIPS, STCNTY[len(ST):], STCNTY[len(ST):], STCNTY[len(ST):]
# fips:
#     counties: STCOFIPS, STCOFIPS, FIPS, FIPS, FIPS
#     tracts: FIPS, FIPS, FIPS, FIPS, FIPS
# state_name:
#     counties: STATE_NAME, FIRST_STATE_NAME, STATE, STATE, STATE
#     tracts: STATE_NAME, STATE_NAME, STATE.strip(), STATE, STATE
# county_name:
#     counties: COUNTY, FIRST_COUNTY, COUNTY, COUNTY, COUNTY
#     tracts: COUNTY, COUNTY, COUNTY.strip(), COUNTY, COUNTY
# svi_edition:
#     counties: 2000, 2010, 2014, 2016, 2018
#     tracts: 2000, 2010, 2014, 2016, 2018
# rank_theme_1
#     counties: USG1TP, R_PL_THEME1, RPL_THEME1, RPL_THEME1, RPL_THEME1
#     tracts: USG1TP, R_PL_THEME1, RPL_THEME1, RPL_THEME1, RPL_THEME1
# rank_theme_2
#     counties: USG2TP, R_PL_THEME2, RPL_THEME2, RPL_THEME2, RPL_THEME2
#     tracts: USG2TP, R_PL_THEME2, RPL_THEME2, RPL_THEME2, RPL_THEME2
# rank_theme_3
#     counties: USG3TP, R_PL_THEME3, RPL_THEME3, RPL_THEME3, RPL_THEME3
#     tracts: USG3TP, R_PL_THEME3, RPL_THEME3, RPL_THEME3, RPL_THEMES3
# rank_theme_4
#     counties: USG3TP, R_PL_THEME4, RPL_THEME4, RPL_THEME4, RPL_THEME4
#     tracts: USG3TP, R_PL_THEME4, RPL_THEME4, RPL_THEME4, RPL_THEME4
# rank_svi
#     counties: USTP, R_PL_THEMES, RPL_THEMES, RPL_THEMES, RPL_THEMES
#     tracts: USTP, R_PL_THEMES, RPL_THEMES, RPL_THEMES, RPL_THEMES
# value_theme_1
#     counties: NA, S_PL_THEME1, SPL_THEME1, SPL_THEME1, SPL_THEME1
#     tracts: NA, S_PL_THEME1, SPL_THEME1, SPL_THEME1, SPL_THEME1
# value_theme_2
#     counties: NA, S_PL_THEME2, SPL_THEME2, SPL_THEME2, SPL_THEME2
#     tracts: NA, S_PL_THEME2, SPL_THEME2, SPL_THEME2, SPL_THEME2
# value_theme_3
#     counties: NA, S_PL_THEME3, SPL_THEME3, SPL_THEME3, SPL_THEME3
#     tracts: NA, S_PL_THEME3, SPL_THEME3, SPL_THEMES3, SPL_THEME3
# value_theme_4
#     counties: NA, S_PL_THEME4, SPL_THEME4, SPL_THEME4, SPL_THEME4
#     tracts: NA, S_PL_THEME4, SPL_THEME4, SPL_THEME4, SPL_THEME4
# value_svi
#     counties: NA, S_PL_THEMES, SPL_THEMES, SPL_THEMES, SPL_THEMES
#     tracts: NA, S_PL_THEMES, SPL_THEMES, SPL_THEMES, SPL_THEMES



from hydrotools._restclient import Alias, AliasGroup

# typing imports
# NOTE: use typing.Literal when minimum python version is 3.8
from typing_extensions import Literal

GeographicScale = Literal["census_tract", "county"]
GeographicContext = Literal["national", "state"]
DataFormat = Literal["csv", "shp"]
Year = Literal["2000", "2010", "2014", "2016", "2018", 2000, 2010, 2014, 2016, 2018]

# Geography options:
# source https://svi.cdc.gov/xmldata/SVI_StatesForDownload.xml
# used by form on https://svi.cdc.gov/data-download-wcms.html

# TODO: add "PuertoRico" and "States". will need to consider what datasets are available for these locations
# LOCATIONS = AliasGroup(
#     [
#         Alias("US", ["us", "united states"]),
#         Alias("Alabama", ["al", "alabama"]),
#         Alias("Alaska", ["ak", "alaska"]),
#         Alias("Arizona", ["az", "arizona"]),
#         Alias("Arkansas", ["ar", "arkansas"]),
#         Alias("California", ["ca", "california"]),
#         Alias("Colorado", ["co", "colorado"]),
#         Alias("Connecticut", ["ct", "connecticut"]),
#         Alias("Delaware", ["de", "delaware"]),
#         Alias("DistrictofColumbia", ["dc", "district of columbia"]),
#         Alias("Florida", ["fl", "florida"]),
#         Alias("Georgia", ["ga", "georgia"]),
#         Alias("Hawaii", ["hi", "hawaii"]),
#         Alias("Idaho", ["id", "idaho"]),
#         Alias("Illinois", ["il", "illinois"]),
#         Alias("Indiana", ["in", "indiana"]),
#         Alias("Iowa", ["ia", "iowa"]),
#         Alias("Kansas", ["ks", "kansas"]),
#         Alias("Kentucky", ["ky", "kentucky"]),
#         Alias("Louisiana", ["la", "louisiana"]),
#         Alias("Maine", ["me", "maine"]),
#         Alias("Maryland", ["md", "maryland"]),
#         Alias("Massachusetts", ["ma", "massachusetts"]),
#         Alias("Michigan", ["mi", "michigan"]),
#         Alias("Minnesota", ["mn", "minnesota"]),
#         Alias("Mississippi", ["ms", "mississippi"]),
#         Alias("Missouri", ["mo", "missouri"]),
#         Alias("Montana", ["mt", "montana"]),
#         Alias("Nebraska", ["ne", "nebraska"]),
#         Alias("Nevada", ["nv", "nevada"]),
#         Alias("NewHampshire", ["nh", "new hampshire"]),
#         Alias("NewJersey", ["nj", "new jersey"]),
#         Alias("NewMexico", ["nm", "new mexico"]),
#         Alias("NewYork", ["ny", "new york"]),
#         Alias("NorthCarolina", ["nc", "north carolina"]),
#         Alias("NorthDakota", ["nd", "north dakota"]),
#         Alias("Ohio", ["oh", "ohio"]),
#         Alias("Oklahoma", ["ok", "oklahoma"]),
#         Alias("Oregon", ["or", "oregon"]),
#         Alias("Pennsylvania", ["pa", "pennsylvania"]),
#         Alias("RhodeIsland", ["ri", "rhode island"]),
#         Alias("SouthCarolina", ["sc", "south carolina"]),
#         Alias("SouthDakota", ["sd", "south dakota"]),
#         Alias("Tennessee", ["tn", "tennessee"]),
#         Alias("Texas", ["tx", "texas"]),
#         Alias("Utah", ["ut", "utah"]),
#         Alias("Vermont", ["vt", "vermont"]),
#         Alias("Virginia", ["va", "virginia"]),
#         Alias("Washington", ["wa", "washington"]),
#         Alias("WestVirginia", ["wv", "west virginia"]),
#         Alias("Wisconsin", ["wi", "wisconsin"]),
#         Alias("Wyoming", ["wy", "wyoming"]),
#     ]
# )

# TODO: remove self refereing alias once bug in AliasGroup.get has been resolved
LOCATIONS = AliasGroup(
    [
        Alias("us", ["us", "united states"]),
        Alias("al", ["al", "alabama"]),
        Alias("ak", ["ak", "alaska"]),
        Alias("az", ["az", "arizona"]),
        Alias("ar", ["ar", "arkansas"]),
        Alias("ca", ["ca", "california"]),
        Alias("co", ["co", "colorado"]),
        Alias("ct", ["ct", "connecticut"]),
        Alias("de", ["de", "delaware"]),
        Alias("dc", ["dc", "district of columbia"]),
        Alias("fl", ["fl", "florida"]),
        Alias("ga", ["ga", "georgia"]),
        Alias("hi", ["hi", "hawaii"]),
        Alias("id", ["id", "idaho"]),
        Alias("il", ["il", "illinois"]),
        Alias("in", ["in", "indiana"]),
        Alias("ia", ["ia", "iowa"]),
        Alias("ks", ["ks", "kansas"]),
        Alias("ky", ["ky", "kentucky"]),
        Alias("la", ["la", "louisiana"]),
        Alias("me", ["me", "maine"]),
        Alias("md", ["md", "maryland"]),
        Alias("ma", ["ma", "massachusetts"]),
        Alias("mi", ["mi", "michigan"]),
        Alias("mn", ["mn", "minnesota"]),
        Alias("ms", ["ms", "mississippi"]),
        Alias("mo", ["mo", "missouri"]),
        Alias("mt", ["mt", "montana"]),
        Alias("ne", ["ne", "nebraska"]),
        Alias("nv", ["nv", "nevada"]),
        Alias("nh", ["nh", "new hampshire"]),
        Alias("nj", ["nj", "new jersey"]),
        Alias("nm", ["nm", "new mexico"]),
        Alias("ny", ["ny", "new york"]),
        Alias("nc", ["nc", "north carolina"]),
        Alias("nd", ["nd", "north dakota"]),
        Alias("oh", ["oh", "ohio"]),
        Alias("ok", ["ok", "oklahoma"]),
        Alias("or", ["or", "oregon"]),
        Alias("pa", ["pa", "pennsylvania"]),
        Alias("ri", ["ri", "rhode island"]),
        Alias("sc", ["sc", "south carolina"]),
        Alias("sd", ["sd", "south dakota"]),
        Alias("tn", ["tn", "tennessee"]),
        Alias("tx", ["tx", "texas"]),
        Alias("ut", ["ut", "utah"]),
        Alias("vt", ["vt", "vermont"]),
        Alias("va", ["va", "virginia"]),
        Alias("wa", ["wa", "washington"]),
        Alias("wv", ["wv", "west virginia"]),
        Alias("wi", ["wi", "wisconsin"]),
        Alias("wy", ["wy", "wyoming"]),
    ]
)
